# Copyright 2021- Le Filament (https://le-filament.com)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html)
from dateutil.relativedelta import relativedelta

from odoo import http
from odoo.http import request

from odoo.addons.portal.controllers.portal import CustomerPortal as CustomerPortal
from odoo.addons.portal.controllers.web import Home as Home
from odoo.addons.web.controllers.utils import is_user_internal


class Home(Home):
    @http.route()
    def index(self, *args, **kw):
        if request.session.uid and not is_user_internal(request.session.uid):
            return request.redirect_query("/operations", query=request.params)
        return super().index(*args, **kw)

    def _login_redirect(self, uid, redirect=None):
        if not redirect and not is_user_internal(uid):
            redirect = "/operations"
        return super()._login_redirect(uid, redirect=redirect)

    @http.route("/login", type="http", auth="none")
    def previous_login(self, s_action=None, db=None, **kw):
        if (
            request.db
            and request.session.uid
            and not is_user_internal(request.session.uid)
        ):
            return request.redirect_query("/operations", query=request.params)
        return request.redirect("/web/login", 303)


class CustomerPortal(CustomerPortal):
    def _get_role(self, operation_id=None):
        role = {
            "isPmo": False,
            "isConsumer": False,
            "isProductor": False,
            "isDataCdc": False,
            "isSuperAdmin": False,
            "isAdmin": False,
        }

        if request.env.user.has_group("oacc.group_operation_superadmin"):
            role["isSuperAdmin"] = True

        if operation_id:
            role.update({"operation": operation_id, "main_object": operation_id})
            if (
                request.env.user.has_group("oacc.group_operation_admin")
                and request.env.user.commercial_partner_id in operation_id.admin_ids
            ):
                role["isAdmin"] = True

            # check if data exists to display tab
            if operation_id.is_cdc_data_exists:
                role["isDataCdc"] = True

            if (
                operation_id.pmo_id.commercial_partner_id.id
                == request.env.user.commercial_partner_id.id
            ):
                role["isPmo"] = True

            if (
                request.env.user.commercial_partner_id.id
                in operation_id.acc_delivery_period_ids.mapped(
                    "partner_id.commercial_partner_id"
                ).ids
            ):
                role["isConsumer"] = True

            if (
                request.env.user.commercial_partner_id.id
                in operation_id.acc_injection_period_ids.mapped(
                    "partner_id.commercial_partner_id"
                ).ids
            ):
                role["isProductor"] = True
        return role

    @http.route()
    def home(self, **kw):
        return request.redirect_query("/operations", query=request.params)

    @http.route(["/infos"], type="http", auth="user", website=True, sitemap=True)
    def infos(self, page=1, **kwargs):
        values = {
            "page_name": "infos",
            "page_breadcrumbs": "Informations",
        }
        return request.render("oacc_portal.infos", values)

    @http.route(
        ["/mentions-legales"], type="http", auth="user", website=True, sitemap=True
    )
    def legal(self, page=1, **kwargs):
        values = {
            "page_name": "legal",
            "page_breadcrumbs": "Mentions légales",
        }
        return request.render("oacc_portal.legal", values)

    @http.route(["/operations"], type="http", auth="user", website=True, sitemap=True)
    def view_all_operations_page(self, page=1, **kwargs):
        """
        Retourne la liste de toutes les opérations dans lesquelles l'utilisateur
        ou sa société sont engagés
        """
        values = self._prepare_portal_layout_values()
        values.update(self._get_role())

        operations = request.env["acc.operation"].search([])

        # Get news
        news = request.env["acc.actu"].search([("operation_id", "=", False)], limit=10)

        values = {
            "operations": operations,
            "news": news,
            "page_name": "operations",
            "page_breadcrumbs": "Mes opérations",
        }

        return request.render("oacc_portal.operations", values)

    def _get_operation_values(self, operation):
        render_values = {}
        render_values.update(self._get_role(operation))

        power_install = sum(
            request.env["acc.counter.period"]
            .sudo()
            .search(
                [
                    ("acc_operation_id", "=", operation.id),
                    ("end_date", "=", False),
                    ("prm_type", "=", "injection"),
                ]
            )
            .mapped("power")
        )

        if render_values.get("isDataCdc"):
            cdc_model = request.env["acc.enedis.cdc"]
            last_record = cdc_model._get_last_cdc_record(operation.id)
            start_date, end_date = cdc_model._get_interval("month", last_record)
            end_date_month = end_date + relativedelta(days=1)
            agg_energy_dict = cdc_model._get_aggregated_energy(
                operation.id, start_date, end_date_month
            )
        else:
            start_date = ""
            agg_energy_dict = {
                "total_prod": 0,
                "total_cons": 0,
                "total_autocons": 0,
            }

        nb_conso = len(
            request.env["acc.counter.period"]
            .sudo()
            .search(
                [
                    ("acc_operation_id", "=", operation.id),
                    ("prm_type", "=", "delivery"),
                ]
            )
            .mapped("acc_counter_id")
        )
        nb_prod = len(
            request.env["acc.counter.period"]
            .sudo()
            .search(
                [
                    ("acc_operation_id", "=", operation.id),
                    ("prm_type", "=", "injection"),
                ]
            )
            .mapped("acc_counter_id")
        )

        # Get news
        news = request.env["acc.actu"].search(
            [("operation_id", "=", operation.id)], limit=10
        )

        render_values.update(
            {
                "news": news,
                "power_install": power_install,
                "consumer_nb": nb_conso,
                "productor_nb": nb_prod,
                "conso_tot": agg_energy_dict["total_cons"],
                "prod_tot": agg_energy_dict["total_prod"],
                "month": start_date,
                "autoprod_rate": round(
                    (agg_energy_dict["total_autocons"] / agg_energy_dict["total_cons"])
                    * 100,
                    2,
                )
                if agg_energy_dict["total_cons"] != 0
                else 0.00,
                "autoconso_rate": round(
                    (agg_energy_dict["total_autocons"] / agg_energy_dict["total_prod"])
                    * 100,
                    2,
                )
                if agg_energy_dict["total_autocons"] != 0
                else 0.00,
                "page_name": "home",
                "page_breadcrumbs": "Accueil",
            }
        )
        return render_values

    @http.route(
        ['/operation/<model("acc.operation"):operation>'],
        type="http",
        auth="user",
        website=True,
    )
    def operation(self, operation, **kw):
        render_values = self._get_operation_values(operation)
        return request.render("oacc_portal.operation_main", render_values)

    @http.route(
        ['/operation/<model("acc.operation"):operation>/contact'],
        type="http",
        auth="user",
        website=True,
    )
    def contact(self, operation, **kw):
        render_values = {}
        render_values.update(self._get_role(operation))
        render_values.update({"page_name": "contact", "page_breadcrumbs": "Contact"})

        return request.render("oacc_portal.operation_contact", render_values)
